uniform extern float4x4 WorldViewProj : WORLDVIEWPROJECTION;
uniform extern float4x4 World : WORLD;
uniform extern float4x4 WorldIT : WORLDINVERSETRANSPOSE;
uniform extern float4x4 ViewI : VIEWINVERSE;
uniform extern float3 DirToLight;

struct VS_OUTPUT
{
    float4 position : POSITION;
    float3 normal : TEXCOORD0;
    float3 view : TEXCOORD1;
};

VS_OUTPUT Transform(
    float4 Pos  : POSITION, 
    float3 Norm : NORMAL )
{
    VS_OUTPUT Out;
    Out.position = mul(Pos, WorldViewProj);
    Out.normal = normalize(mul(Norm, WorldIT));
    float3 worldEyePos = ViewI[3].xyz;
    float3 worldVertPos = mul(Pos, World).xyz;
    Out.view = worldEyePos - worldVertPos; // nicht normalisieren, sonst falsche Interpolation
    return Out;
}

float4 PixelShader( VS_OUTPUT vsout ) : COLOR
{
	float3 normal = normalize(vsout.normal);
	float3 view = normalize(vsout.view);
	
	// Die HLSL-Funktion "lit" kann alles dies in einer Zeile.
	float3 halfvec = normalize(view + DirToLight);
	float specular = dot(normal, halfvec);
	if(specular < 0)
	{
		specular = 0;
	}
	float diffuse = dot(normal, DirToLight);
	if(diffuse < 0)
	{
		diffuse = 0;
		specular = 0;
	}
    return diffuse*float4(1,0,0,1) + pow(specular, 100.0)*float4(1,1,1,1);
}

technique phong
{
    pass P0
    {
        vertexShader = compile vs_2_0 Transform();
        pixelShader = compile ps_2_0 PixelShader();
    }
}